@@ -106,7 +106,7 @@ def _diff_graph_list(n1, n2, tree_full1, tree_full2):
                 for child in affected_hyperparam_subtree:
                     cost += 1 # FIXME
                     operations.append((child, ["DELETE", tree_full1[child]["label"], ""]))
-        else:
+        elif op == "UPDATE":
             name1 = operation[0][0]
             name2 = operation[0][1]
             label1 = operation[1][1]
@@ -136,6 +136,8 @@ def _diff_graph_list(n1, n2, tree_full1, tree_full2):
                             cost += 1 # FIXME
                             _operation = (child, ["INSERT", "", tree_full2[child]["label"]])
                             operations.append(_operation)
+        else:
+            operations.append(operation)
 
     return cost, operations
 
@@ -255,16 +257,35 @@ from alex.annotators import param_count
 def diff(network_config_1,
          network_config_2,
          render_to=None,
-         dpi=800, lazy=False):
-    graph_list1 = dsl_parser.parse(network_config_1, lazy=lazy)
-    graph_list2 = dsl_parser.parse(network_config_2, lazy=lazy)
+         dpi=800, lazy=False, merge_blocks=[]):
+    if not isinstance(network_config_1, list):
+        graph_list1 = dsl_parser.parse(network_config_1, lazy=lazy)
+    else:
+        graph_list1 = network_config_1
+    if not isinstance(network_config_2, list):
+        graph_list2 = dsl_parser.parse(network_config_2, lazy=lazy)
+    else:
+        graph_list2 = network_config_2
     tree_full1 = param_count.ParamCount(graph_list1).annotate_tree()
     tree_full2 = param_count.ParamCount(graph_list2).annotate_tree()
 
     operations = []
     cost = 0
-    for block in registry.BLOCKS:
 
+    graph_list1_block = []
+    graph_list2_block = []
+    for block in merge_blocks:
+        graph_list1_block += list(filter(lambda x: x["meta"]["block"]==block, graph_list1))
+        graph_list2_block += list(filter(lambda x: x["meta"]["block"]==block, graph_list2))
+    _cost, _operations = _diff_graph_list(graph_list1_block,
+                                          graph_list2_block,
+                                          tree_full1, tree_full2)
+    cost += _cost
+    operations += _operations
+
+    for block in registry.BLOCKS:
+        if block in merge_blocks:
+            continue
         graph_list1_block = list(filter(lambda x: x["meta"]["block"]==block, graph_list1))
         graph_list2_block = list(filter(lambda x: x["meta"]["block"]==block, graph_list2))
         _cost, _operations = _diff_graph_list(graph_list1_block,
@@ -336,7 +357,7 @@ def matched_ingredients(network_config_1,
                                    naive=False).annotate_tree()
     tree2 = param_count.ParamCount(network_config_2,
                                    naive=False).annotate_tree()
-    _, operations = ted(tree1, tree2)
+    _, operations = diff(network_config_1, network_config_2, merge_blocks=["model_block", "loss_block"])
     if render_to is not None:
         print("Done computing diff. Rendering image")
         annotation = annotate_ops(operations)
@@ -348,7 +369,7 @@ def matched_ingredients(network_config_1,
         axs[1].imshow(img2)
         axs[1].axis("off")
         fig.tight_layout()
-        if render_to != "":
+        if render_to != "" or render_to is None:
             fig.savefig(render_to, dpi="figure")
             print("Diff images written to: %s" % render_to)
         else:
