@@ -78,6 +78,7 @@ def alex_reader(user_defined):
               const.INPUTS: _read_inputs,
               const.HYPERPARAMS: _read_hyperparams,
               "visible": lambda x: {**{"visible": True}, **x} if "visible" not in x else clone(x),
+              "probe": lambda x: {**{"probe": False}, **x} if "probe" not in x else clone(x),
               "trainable": lambda x: {**{"trainable": None}, **x} if "trainable" not in x else clone(x),
               "dtype": lambda x: {**{"dtype": default["dtype"]}, **x} if "dtype" not in x else clone(x)}
     return reader
@@ -208,6 +209,7 @@ def _get_node(component):
             graph[const.META][m] = component[m]
     graph[const.META][const.NAME] = None
     graph[const.META]["visible"] = component["visible"]
+    graph[const.META]["probe"] = component["probe"]
     graph[const.META]["trainable"] = component["trainable"]
     graph[const.META]["block"] = component["block"]
     graph[const.META]["dtype"] = component["dtype"]
@@ -876,9 +878,12 @@ def annotate(components, lazy=True):
                 validator.validate_connection(component, components)
 
             inputs = component["meta"]["inputs"]
-        except Exception:
-            logger.error("Component %s annotation failed" % (name))
-            raise Exception
+        except Exception as err:
+            msg = "Component %s annotation failed. \n" % (name)
+            msg += str(err)
+            logger.error(msg)
+            logger.error(str(err))
+            raise Exception(msg)
         components[name] = component
     return list(components.values())
 
@@ -899,7 +904,14 @@ def parse(yml_file, return_dict=False, lazy=True):
     prev_component = None
     try:
         # Read the yml file
-        user_defined = util.read_yaml(yml_file)
+        if isinstance(yml_file, str):
+            user_defined = util.read_yaml(yml_file)
+        elif isinstance(yml_file, dict):
+            user_defined = deepcopy(yml_file)
+        else:
+            msg = "First argument has to be a path or a dict type"
+            logger.error(msg)
+            raise TypeError(msg)
         reader = alex_reader(user_defined)
         # Step 1: transform yml into a (recursive) dict
         # Each node in the dict has the following keys:
@@ -936,10 +948,11 @@ def parse(yml_file, return_dict=False, lazy=True):
         # if config_to_type(yml_file) not in const.RECIPE_TYPES:
         graph = annotate(graph, lazy=lazy) # json schema
         # Step 5: load graph and states from checkpoint; check if the structure matches the one defined in the DSL
-    except Exception:
-        message = "Error during parsing configuration %s" % yml_file
-        logger.error(message)
-        raise Exception
+    except Exception as err:
+        msg = "Error during parsing configuration %s" % yml_file
+        msg += "\n %s" % str(err)
+        logger.error(msg)
+        raise Exception(str(err))
     if return_dict:
         graph = list_to_dict(graph)
     return graph
