@@ -16,7 +16,7 @@ TENSOR_SHAPE = {"input_shape": {"tf": ["inputs.get_shape().as_list()", []]}}
 
 AS_TENSOR = {"as_tensor": {"tf": ["tf.convert_to_tensor",
                                   {"value": "np.asarray(inputs)",
-                                   "dtype": "tf_dtypes['dtype']",
+                                   "dtype": "tf_dtypes[dtype]",
                                    "dtype_hint": "None"}],
                            "pytorch": ["torch.as_tensor",
                                        {"data": "np.asarray(inputs)",
@@ -371,21 +371,34 @@ MODEL_INGREDIENTS = {**STATEFUL_INGREDIENTS,
 MODEL_BLOCK = {**MODEL_INGREDIENTS, **UTILITIES}
 
 # -------------------------------- Loss -------------------------------------- #
-LOSSES = {"cross_entropy": {"tf": ["tf.nn.softmax_cross_entropy_with_logits",
-                                   {"labels": "inputs[0]",
-                                    "logits": "inputs[1]",
-                                    "axis": "-1",
-                                    "name": "name"}],
-                            "pytorch": ["torch.nn.functional.cross_entropy",
-                                        {"weight": "None",
-                                         "ignore_index": "-100",
-                                         "reduction": "'mean'",
-                                         "target": "inputs[0]",
-                                         "input": "inputs[1]"}],
-                            "alex": ["cross_entropy",
-                                     {"name": True,
-                                      "dtype": True,
-                                      "training": False}]}}
+CLASSIFICATION_LOSSES = {"cross_entropy": {"tf": ["tf.nn.softmax_cross_entropy_with_logits",
+                                                  {"labels": "inputs[0]",
+                                                   "logits": "inputs[1]",
+                                                   "axis": "-1",
+                                                   "name": "name"}],
+                                           "pytorch": ["torch.nn.functional.cross_entropy",
+                                                       {"weight": "None",
+                                                        "ignore_index": "-100",
+                                                        "reduction": "'mean'",
+                                                        "target": "inputs[0]",
+                                                        "input": "inputs[1]"}],
+                                           "alex": ["cross_entropy",
+                                                    {"name": True,
+                                                     "inputs": False,
+                                                     "dtype": True,
+                                                     "training": False}]}}
+REGRESSION_LOSSES = {"mse": {"pytorch": ["torch.nn.functional.mse_loss",
+                              {"input": "inputs[0]",
+                               "target": "inputs[1]",
+                               "size_average": "None",
+                               "reduce": "None",
+                               "reduction": "'mean'"}],
+                             "alex": ["mse", {"inputs": False}],
+                             "tf": ["tf.keras.metrics.mean_squared_error",
+                                    {"y_true": "inputs[0]",
+                                     "y_pred": "inputs[1]"}]}}
+LOSSES = {**CLASSIFICATION_LOSSES, **REGRESSION_LOSSES}
+
 # TODO: bitwise, l2, mean_square
 REGULARIZERS = {"regularizer_l2": {"tf": ["coeff*sum(list(map(lambda x: tf.nn.l2_loss(t=trainable_params[x], name=name), inputs)))",
                                           {}],
@@ -513,6 +526,12 @@ TYPES = {0: "STATELESS_INGREDIENTS_WITHOUT_HYPE",
          2: "STATEFUL_INGREDIENTS"}
 
 
+def get_trainable_params_list(params):
+    trainable_params_list = []
+    for tv in params:
+        if tv.split('/')[-1] in ALL_TRAINABLE_PARAMS:
+            trainable_params_list.append(params[tv])
+    return trainable_params_list
 # -------------------------------------------------------------------------- #
 # Summary
 # -------------------------------------------------------------------------- #
