@@ -13,31 +13,17 @@ trainset = torchvision.datasets.CIFAR10(root='./data', train=True,
 trainloader = torch.utils.data.DataLoader(trainset, batch_size=100,
                                           shuffle=True, num_workers=2)
 
-testset = torchvision.datasets.CIFAR10(root='./data', train=False,
+valset = torchvision.datasets.CIFAR10(root='./data', train=False,
                                        download=True, transform=transform)
-testloader = torch.utils.data.DataLoader(testset, batch_size=10000,
+valloader = torch.utils.data.DataLoader(valset, batch_size=1000,
                                          shuffle=False, num_workers=2)
 
 classes = ('plane', 'car', 'bird', 'cat',
            'deer', 'dog', 'frog', 'horse', 'ship', 'truck')
 
+val_inputs, val_labels = iter(valloader).next()
 
-def imshow(img):
-    img = img / 2 + 0.5     # unnormalize
-    npimg = img.numpy()
-    plt.imshow(np.transpose(npimg, (1, 2, 0)))
-    plt.show()
+val_inputs = val_inputs.to(device)
+val_labels = val_labels.to(device)
 
-# get some random training images
-dataiter = iter(trainloader)
-images, labels = dataiter.next()
-inputs = images.to(device)
-print(device)
-# show images
-# imshow(torchvision.utils.make_grid(images))
-# print labels
-# print(' '.join('%5s' % classes[labels[j]] for j in range(4)))
-
-test_inputs, test_labels = iter(testloader).next()
-
-loop(trainloader, test_inputs, test_labels)
+loop(probes, trainable_params, val_inputs)
\ No newline at end of file
