@@ -12,6 +12,9 @@ from alex.engine import ns_alex
 ENGINE = "pytorch"
 DIM_ORDER = ["batch_size", "channel", "height", "width"]
 
+DEFINED = {"torch", "np", "torch_types", "device"}
+
+# LOOP_ARGS = ["trainloader", "val_inputs", "val_labels"]
 
 def add_imports(additional_modules=[]):
     default_modules = [["torch"],
@@ -42,21 +45,20 @@ def add_code_block(code, block):
         code.append(ns_alex.indent(block.replace("\n", "\n\t"), levels=1))
 
 
-def wrap_in_class(trainable_params_code, component_code, loss_code, optimizer_code, scheduler_code=""):
+def wrap_in_class(trainable_params_code, component_code, loss_code, optimizer_code, scheduler_code="", load_from_ckpt=False, trainable_params_args=[], model_args=[]):
     code = []
     code.append("class Model(torch.nn.Module):")
     code.append("")
     code.append(ns_alex.indent("def __init__(self, ckpt=None):", levels=1))
     code.append(ns_alex.indent("super(Model, self).__init__()", levels=2))
-    code.append(ns_alex.indent("self.%s = self.get_trainable_params(ckpt)" % (const.ALEX_ARG_TRAINABLE_PARAMS), levels=2))
+    code.append(ns_alex.indent("self.%s = self.get_trainable_params(%s)" % (const.ALEX_ARG_TRAINABLE_PARAMS, ", ".join(trainable_params_args)), levels=2))
     code.append(ns_alex.indent("self.params = []", levels=2))
     code.append(ns_alex.indent("for var in self.%s:" % const.ALEX_ARG_TRAINABLE_PARAMS, levels=2))
     code.append(ns_alex.indent("self.register_parameter(var, self.%s[var])" % const.ALEX_ARG_TRAINABLE_PARAMS, levels=3))
     code.append(ns_alex.indent("self.params.append({'params': self.%s[var]})" % const.ALEX_ARG_TRAINABLE_PARAMS, levels=3))
     code.append("")
-    code.append(ns_alex.indent("def forward(self, x, training):", levels=1))
-    code.append(ns_alex.indent("x = self.model(x, self.%s, training)" % const.ALEX_ARG_TRAINABLE_PARAMS,
-                               levels=2))
+    code.append(ns_alex.indent("def forward(self, %s):" % (", ".join(model_args)), levels=1))
+    code.append(ns_alex.indent("x = self.model(%s)" % (", ".join(model_args)), levels=2))
     code.append(ns_alex.indent("return x", levels=2))
     code.append("")
     add_code_block(code, trainable_params_code)
@@ -69,13 +71,11 @@ def wrap_in_class(trainable_params_code, component_code, loss_code, optimizer_co
 
 
 # Boilerplates:
-def instantiate(config, load_from, save_to):
+def instantiate(config, engine, load_from, save_to, **kwargs):
     return """
 from alex.alex.checkpoint import Checkpoint
 
-C = Checkpoint("%s",
-               %s,
-               %s)
+C = Checkpoint("%s", '%s', %s, %s)
 
 ckpt = C.load()
 
@@ -83,61 +83,112 @@ model = Model(ckpt)
 
 model.to(device)
 
+trainable_params = model.trainable_params
 optimizer = model.get_optimizer(model.params)
 
 learning_rate = model.get_scheduler(optimizer)
 
-""" % (config, str(load_from), str(save_to))
+probes = dict()
+""" % (config, engine, str(load_from), str(save_to))
 
 
-def training(save_to):
+def loop(save_to, train_args, evaluation_args):
     if save_to:
         save_str = "C.save(model.trainable_params)"
     else:
         save_str = ""
+    func_name = "loop"
     return \
-    """
-def loop(trainloader, val_inputs, val_labels):
-    for epoch in range(90):
+    func_name, """
+for epoch in range(90):
+    i = 0
+    for data in trainloader:
+        inputs, labels = data
 
-        for i, data in enumerate(trainloader, 0):
-
-            inputs, labels = data
-
-            inputs = inputs.to(device)
-            labels = labels.to(device)
-            optimizer.zero_grad()
-
-            output = model(inputs, True)
-            loss = model.get_loss(model.trainable_params, [labels, output])
-            loss.backward()
-            optimizer.step()
-
-            if i %% 500 == 499:
-                accuracy, loss_val = validation(val_inputs, val_labels)
-                print('[%%d, %%d, 500] accuracy: %%.3f, loss: %%.3f' %%
-                      (epoch, i, accuracy, loss_val))
-                %s
-        learning_rate.step()
-    print('Finished Training')
-""" % save_str
-
-
-def validation():
-    return """
-def validation(inputs, labels):
-    with torch.no_grad():
         inputs = inputs.to(device)
         labels = labels.to(device)
+        train(%s)
+        optimizer.step()
 
-        outputs = model(inputs, False)
-        _, predicted = torch.max(outputs.data, 1)
-        total = labels.size(0)
-        correct = (predicted == labels).sum().item()
-        loss = model.get_loss(model.trainable_params, [labels, outputs])
-    return correct / total, loss
+        if i %% 500 == 499:
+            results = evaluation(%s)
+            print("Epoch:", epoch, results)
+            %s
+        i += 1
+    learning_rate.step()
+print('Finished Training')
+""" % (", ".join(train_args).replace("data_block_input_data", "inputs").replace("data_block_labels", "labels"),
+       ", ".join(evaluation_args).replace("inputs", "val_inputs").replace("labels", "val_labels"),
+       save_str)
 
+
+def train(model_args, loss_args):
+    func_name = "train"
+    return \
+    func_name, """
+optimizer.zero_grad()
+model.training=True
+training = model.training
+preds = model(%s)
+loss = model.get_loss(%s)
+loss.backward()
+""" % (", ".join(model_args),
+       ", ".join(loss_args).replace("model_block_output", "preds"))
+
+
+def inference(model_args, mode):
+    code_str = """
+model.training=False
+training = model.training
 """
+    if mode == "classification":
+        code_str += """
+preds = model(%s)
+classes = torch.max(preds, 1)[1]
+""" % (", ".join(model_args))
+        return_str = "return preds, classes"
+    elif mode == "regression":
+        code_str += """
+preds = model(%s)\n
+""" % (", ".join(model_args))
+        return_str = "return preds"
+    code_str += return_str
+    func_name = "inference"
+    return func_name, code_str
+
+
+def evaluation(inference_args, loss_args, mode="classification"):
+    evaluation_str = """
+model.training=False
+training = model.training
+"""
+    if mode == "classification":
+        evaluation_str += """
+gt = labels
+total = gt.size(0)
+preds = results[0]
+classes = results[1]
+matches = (classes == gt).sum().item()
+perf = matches / total
+"""
+        inference_str = "results = inference(%s)\n" % (", ".join(inference_args).replace("data_block_labels", "labels").replace("data_block_input_data", "inputs"))
+        return_str = "return perf, loss"
+    else:
+        evaluation_str += ""
+        inference_str = "preds = inference(%s)\n" % (", ".join(inference_args).replace("data_block_labels", "labels").replace("data_block_input_data", "inputs"))
+        return_str = "return loss"
+
+    func_name = "evaluation"
+    code_str = inference_str + """
+%s
+loss = model.get_loss(%s)
+%s
+""" % (evaluation_str,
+       ", ".join(loss_args).replace("model_block_output", "preds").replace("data_block_labels", "labels").replace("data_block_input_data", "inputs"),
+       return_str)
+
+    return func_name, code_str
+
 
 ## User defined layers:
 
